@@ -1,73 +1,3 @@
-      run_EMstep <- function(data=NULL, gpar=NULL, loglik=NULL, maxit=NULL, N=NULL, p=NULL, G=NULL, q=NULL, epsilon=NULL, label=NULL){
-      counter =0
-      mu = matrix(0, nrow=G, ncol=p)
-      phi= matrix(0, nrow=G, ncol=p)
-      alpha= matrix(0, nrow=G, ncol=p)
-      cplr = matrix(0, nrow=G, ncol=p*2)
-      gamr = matrix(0, nrow=G, ncol=p*p)
-#      sigmar = matrix(0, nrow=G, ncol=p*p)
-      cpl0 =matrix(0, nrow=G, ncol=2)
-      wg = matrix(0, nrow=G, ncol=2)
-      for(k in 1:G){
-      par = gpar[[k]]
-      mu[k,] = par$mu
-      phi[k,] = par$phi
-      alpha[k,] = par$alpha
-      cplr[k,] = c(t(par$cpl))
-      gamr[k,] = c(t(par$gam))
-#      sigmar[k,] = c(t(par$sigma))
-      cpl0[k,] = par$cpl0
-      wg[k,] = par$wg
-      }
-      pi = gpar$pi
-      v = 1
-      mu1= t(mu)
-      phi1=t(phi)
-#      print("phi1")
-#      print(phi1)
-      alpha1=t(alpha)
-#      print("alpha1")
-#      print(alpha1)
-      cpl1=t(cplr)
-#      print("cplr")
-#      print(cplr)
-      gam1=t(gamr)
-#      sigma1=t(sigmar)
-      cpl01=t(cpl0)
-      wg1=t(wg)
-      if(is.null(label) ==T) label=rep(0,N)
-      EM <- .C("EMstep", as.double(mu1), as.double(phi1), as.double(alpha1),
-#            as.double(cpl1), as.double(gam1), as.double(sigma1), as.double(cpl01)
-            as.double(cpl1), as.double(gam1), as.double(cpl01)
-            , as.double(wg1), as.double(data), as.double(loglik), as.integer(maxit),
-            as.integer(N), as.integer(p), as.integer(G), as.double(epsilon), as.integer(label),
-            as.integer(v), as.double(pi), as.integer(counter), PACKAGE="MixGHD")
-      loglik = EM[[9]]
-      mu = matrix(EM[[1]], nrow=G,ncol=p, byrow=TRUE)
-      phi = matrix(EM[[2]], nrow=G, ncol=p, byrow=TRUE)
-      alpha = matrix(EM[[3]], nrow=G, ncol=p, byrow=TRUE)
-      counter = EM[[18]]
-      cpl = array(EM[[4]], dim=c(p,2,G))
-      gam = array(EM[[5]], dim=c(p,p,G))
-#      sigma = array(EM[[6]], dim=c(p,p,G))
-      cpl0 = matrix(EM[[6]], nrow=G, ncol=2, byrow=TRUE)
-      wg = matrix(EM[[7]], nrow=G, ncol=2, byrow=TRUE)
-      gpar = list()
-      for(k in 1:G){
-         gpar[[k]] = list()
-         gpar[[k]]$mu = mu[k,]
-         gpar[[k]]$phi = phi[k,]
-         gpar[[k]]$alpha = alpha[k,]
-         gpar[[k]]$cpl = matrix(cpl[,,k],nrow=p, ncol=2,  byrow=TRUE)
-         gpar[[k]]$gam = matrix(gam[,,k],nrow=p, ncol=p,  byrow=TRUE)
-#         gpar[[k]]$sigma = matrix(sigma[,,k],nrow=p, ncol=p,  byrow=TRUE)
-         gpar[[k]]$cpl0 = cpl0[k,]
-         gpar[[k]]$wg = wg[k,]
-         }
-      gpar$pi = EM[[17]]
-      val = list(loglik,gpar,counter)
-      return(val)
-     }
 MainMCGHD=function(data=NULL, gpar0=NULL, G=2, max.iter=100, eps=1e-2,  label=NULL, method="km",nr=NULL){
     pcol=ncol(data)
 
@@ -106,20 +36,21 @@ MainMCGHD=function(data=NULL, gpar0=NULL, G=2, max.iter=100, eps=1e-2,  label=NU
         gpar = EMgrstep(data=data, gpar=gpar, v=1, label = label,it=i)
         loglik[i] = llik(data, gpar)
     }
-       N = nrow(data)
-       p = ncol(data)
-       maxit = max.iter
-       temp <- run_EMstep(data, gpar, loglik, maxit, N, p, G, q, eps, label)
-     i = temp[[3]]
-#    if(i<max.iter){loglik=loglik[-(i+1:max.iter)]}
-    if(i<max.iter){temp[[1]]=temp[[1]][-(i+1:max.iter)]}  
-    BIC=2*temp[[1]][i]-log(nrow(data))*((G-1)+G*(4*pcol+2+pcol*(pcol-1)/2))
-    AIC=2*temp[[1]][i]-2*((G-1)+G*(4*pcol+2+pcol*(pcol-1)/2))
-    AIC3=2*temp[[1]][i]-3*((G-1)+G*(4*pcol+2+pcol*(pcol-1)/2))
-    z=weights(data=data, gpar= temp[[2]])
+    while ( ( getall(loglik[1:i]) > eps) & (i < (max.iter) ) )  {
+      i = i+1
+      gpar = EMgrstep(data=data, gpar=gpar, v=1, label = label,it=i)
+      loglik[i] = llik(data, gpar)
+      
+    }
+  if(i<max.iter){loglik=loglik[-(i+1:max.iter)]}
+    BIC=2*loglik[i]-log(nrow(data))*((G-1)+G*(4*pcol+pcol*(pcol-1)/2))
+    z=weightsMS(data=data, gpar= gpar)
+    map=MAPMS(data=data, gpar= gpar, label=label)
     ICL=BIC+2*sum(log(apply(z,1,max)))
-    par=partrue(temp[[2]],G)
-    val = list(loglik= temp[[1]][1:i], gpar=temp[[2]],par=par, z=z, map=MAP(data=data, gpar= temp[[2]], label=label),BIC=BIC,ICL=ICL,AIC=AIC,AIC3=AIC3 )
+    AIC=2*loglik[i]-2*((G-1)+G*(4*pcol+pcol*(pcol-1)/2))
+    AIC3=2*loglik[i]-3*((G-1)+G*(4*pcol+pcol*(pcol-1)/2))
+    par=partrue(gpar,G)
+    val = list(loglik= loglik, gpar=gpar, z=z, map=map,par=par, BIC=BIC,ICL=ICL,AIC=AIC,AIC3=AIC3)
     return(val)
 
 }
